import socket
from SECRET import FLAG

IRC_SERVER = "127.0.0.1"
IRC_PORT = 6667
NICKNAME = "Chatter9000"
REALNAME = "ChatBotSSM"

correctHash = 24556
authorizedUsers = set()

def connect_to_irc():
    irc_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    irc_socket.connect((IRC_SERVER, IRC_PORT))

    irc_socket.send(f"NICK {NICKNAME}\r\n".encode())
    irc_socket.send(f"USER {NICKNAME} 0 * :{REALNAME}\r\n".encode())

    while True:
        response = irc_socket.recv(2048).decode("utf-8", errors="ignore")
        if "PING" in response:
            irc_socket.send(f"PONG {response.split()[1]}\r\n".encode())
        
        if f"PRIVMSG {NICKNAME}" in response:
            msg = response.split(":", 2)[2]
            print(f"Message recived: {msg}")

            sender = response.split("!", 1)[0][1:]
            command, args = parse_message(response)

            if command:
                handle_commands(command, args, sender, irc_socket)

def parse_message(message):
    parts = message.split(":", 2)
    if len(parts) < 3:
        return None, None
    
    user_message = parts[2].strip()
    if not user_message.startswith("!"):
        return None, None
    
    command_parts = user_message.split(" ", 1)
    command = command_parts[0]
    args = command_parts[1] if len(command_parts) > 1 else None

    return command, args

def handle_commands(command, args, sender, irc_socket):
    match command:
        case "!help":
            send_message(irc_socket, sender, "Normal commands: !help, !hello, !auth <hash> Admin commands: !get_flag, !hack_fra")
        case "!hello":
            send_message(irc_socket, sender, f"HI {sender} :3")
        case "!auth":
            if {sender} in authorizedUsers:
                send_message(irc_socket, sender, "You are already authenticated")
            elif best_hash_ever(args) == correctHash:
                authorizedUsers.add(sender)
                send_message(irc_socket, sender, "You are now authenticated")
            else:
                send_message(irc_socket, sender, "Wrong hash")
        case "!get_flag":
            if sender in authorizedUsers:
                send_message(irc_socket, sender, FLAG)
            else:
                send_message(irc_socket, sender, "You are not authenticated")
        case "!hack_fra":
            if sender in authorizedUsers:
                send_message(irc_socket, sender, "Nuhu")
            else:
                send_message(irc_socket, sender, "You are not authenticated")
        case _:
            send_message(irc_socket, sender, f"Unknown command: {command}")

def send_message(irc_socket, target, message):
    irc_socket.send(f"PRIVMSG {target} :{message}\r\n".encode())

def best_hash_ever(input):
    #This has is the best (trust trust)
    if isinstance(input, str) and len(input) > 3:
        return sum((i + 1) * ord(c) for i, c in enumerate(input))
    else:
        print(f"Expected a string, but got {type(input)}")
        return 0

if __name__ == "__main__":
    connect_to_irc()
